@@ -2,6 +2,7 @@ package main
 
 import (
 	"bytes"
+	"encoding/json"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -13,6 +14,7 @@ import (
 	"github.com/multiprocessio/datastation/runner"
 
 	"github.com/google/uuid"
+	"github.com/olekukonko/tablewriter"
 )
 
 func resolveContentType(fileExtensionOrContentType string) runner.MimeType {
@@ -45,20 +47,15 @@ func evalFileInto(file string, out *os.File) error {
 func getShape(resultFile, panelId string) *runner.Shape {
 	s, err := runner.ShapeFromFile(resultFile, panelId, 10_000, 100)
 	if err != nil {
-		switch t := err.(type) {
-		case *runner.DSError:
-			if t.Name == "NotAnArrayOfObjectsError" {
-				rest := "."
-				if panelId != "" {
-					rest = ": " + panelId + "."
-				}
-				log.Fatalf("Input is not an array of objects%s", rest)
-			}
+		log.Fatal(err)
+	}
 
-			log.Fatal(err)
-		default:
-			log.Fatal(err)
+	if !runner.ShapeIsObjectArray(*s) {
+		rest := "."
+		if panelId != "" {
+			rest = ": " + panelId + "."
 		}
+		log.Fatalf("Input is not an array of objects%s", rest)
 	}
 
 	return s
@@ -97,6 +94,7 @@ func main() {
 	runner.Verbose = false
 	var nonFlagArgs []string
 	stdin := false
+	pretty := false
 	for _, arg := range os.Args[1:] {
 		if arg == "-v" || arg == "--verbose" {
 			runner.Verbose = true
@@ -113,6 +111,11 @@ func main() {
 			return
 		}
 
+		if arg == "-p" || arg == "--pretty" {
+			pretty = true
+			continue
+		}
+
 		nonFlagArgs = append(nonFlagArgs, arg)
 	}
 
@@ -234,11 +237,10 @@ func main() {
 		return
 	}
 
-	connector, tmp, err := runner.MakeTmpSQLiteConnector()
+	connector, err := runner.MakeTmpSQLiteConnector()
 	if err != nil {
 		log.Fatal(err)
 	}
-	defer os.Remove(tmp.Name())
 	project.Connectors = append(project.Connectors, *connector)
 
 	query := lastNonFlagArg
@@ -263,15 +265,62 @@ func main() {
 		log.Fatal(err)
 	}
 
-	// Dump the result to stdout
-	fd, err := os.Open(runner.GetPanelResultsFile(project.Id, panel.Id))
+	resultFile := runner.GetPanelResultsFile(project.Id, panel.Id)
+	fd, err := os.Open(resultFile)
 	if err != nil {
 		log.Fatalf("Could not open results file: %s", err)
 	}
 
-	_, err = io.Copy(os.Stdout, fd)
+	if !pretty {
+		// Dump the result to stdout
+		_, err = io.Copy(os.Stdout, fd)
+		if err != nil {
+			log.Fatal(err)
+		}
+		fmt.Println()
+		return
+	}
+
+	s, err := runner.ShapeFromFile(resultFile, "results", 10_000, 100)
 	if err != nil {
 		log.Fatal(err)
 	}
-	fmt.Println()
+
+	var columns []string
+	for name := range s.ArrayShape.Children.ObjectShape.Children {
+		columns = append(columns, name)
+	}
+
+	table := tablewriter.NewWriter(os.Stdout)
+	table.SetHeader(columns)
+	table.SetAutoFormatHeaders(false)
+
+	dec := json.NewDecoder(fd)
+	var rows []map[string]interface{}
+	err = dec.Decode(&rows)
+	if err != nil {
+		log.Fatal(err)
+	}
+
+	for _, objRow := range rows {
+		var row []string
+		for _, column := range columns {
+			var cell string
+			switch t := objRow[column].(type) {
+			case bool, byte, complex64, complex128, error, float32, float64,
+				int, int8, int16, int32, int64,
+				uint, uint16, uint32, uint64, uintptr:
+				cell = fmt.Sprintf("%#v", t)
+			case string:
+				cell = t
+			default:
+				cellBytes, _ := json.Marshal(t)
+				cell = string(cellBytes)
+			}
+			row = append(row, cell)
+		}
+		table.Append(row)
+	}
+
+	table.Render()
 }
